from typing import Set, List
from unittest.mock import Mock
from hypothesis import given
from hypothesis.strategies import sets, integers, lists, dictionaries, characters, randoms
from kiss_cache import Cache


class TestCache:

    @given(sets(integers(), min_size=1, max_size=1000))
    def test_cache_keys_length(self, x: Set[int]):
        cache = Cache()

        @cache.memoize()
        def cached_func(val):
            return val

        for i in x:
            cached_func(i)

        assert len(cache.store.store.keys()) == len(x)

    @given(lists(integers(), min_size=1, max_size=1000))
    def test_body_execution(self, x: List[int]):
        cache = Cache()

        mock = Mock()

        @cache.memoize()
        def cached_func(val):
            mock()
            return val

        for i in x:
            cached_func(i)

        assert mock.call_count == len(set(x))

    @given(dictionaries(characters(), randoms()))
    def test_cache_key_generation(self, x: dict):
        cache = Cache()

        print(x)

        @cache.memoize()
        def cached_func(*args, **kwargs):
            return args, kwargs

        cached_func(**x)

        print(set(x.keys()))

        assert len(cache.store.store.keys()) == len(set(x.keys()) or 1)



import logging
from typing import Callable, Any

logger = logging.getLogger(__name__)


def default_serialize(value: Any) -> str:
    return value


def default_deserialize(value: str) -> Any:
    return value


class DjangoCacheStore:

    def __init__(self, cache_identifier='default',
                 serialize: Callable[[Any], str] = default_serialize,
                 deserialize: Callable[[str], Any] = default_deserialize):

        try:
            from django.core.cache import caches
        except ImportError:
            raise ImportError('Django required for this cache store')

        self.cache = caches[cache_identifier]
        self.serialize = serialize
        self.deserialize = deserialize

    def get(self, key):

        try:
            value = self.cache.get(key)

            if value is None:
                return None

            return self.deserialize(value)

        except Exception as e:
            logger.exception(e)
            return None

    def set(self, key, value, expire):
        try:
            value = self.serialize(value)
            return self.cache.set(key, value, expire)
        except Exception as e:
            logger.exception(e)



from time import time
from typing import Dict, Any


class InMemoryPackage:
    __slots__ = ['value', 'expired']

    def __init__(self, value: Any, expired: float):
        self.value = value
        self.expired = expired


class InMemoryStore:

    def __init__(self):
        self.store: Dict[str, InMemoryPackage] = {}

    def get(self, key: str):
        package = self.store.get(key)

        if package is None:
            return None

        if time() > package.expired:
            self.store.pop(key)
            return None

        return package.value

    def set(self, key: str, value: Any, expire):

        package = InMemoryPackage(value=value, expired=time() + expire)

        self.store[key] = package
        return True



from setuptools import find_packages, setup

setup(
    name='kiss_cache',
    packages=find_packages(),
    version='0.5.9',
    license='MIT',
    description='"Keep It Simple Stupid" Cache',
    author='Bergutov Ruslan',
    author_email='ruslanbergutov@gmail.com',
    url='https://github.com/HiveTraum/kiss_cache',
    download_url='https://github.com/HiveTraum/kiss_cache/archive/0.5.9.tar.gz',
    keywords=['cache', 'memoize', 'kiss'],
    install_requires=[],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
